﻿using OAuth2Authorizer.Framework;
using OAuth2Authorizer.Serialization;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Json;
using System.Text;

namespace OAuth2Authorizer.Facebook
{
    internal class FacebookAuthorizer : AuthorizerBase, IAuthorizer
    {
        private static readonly string AuthorizeUrl = "https://www.facebook.com/dialog/oauth";
        private static readonly string AuthorizeUrlTemplate = "{0}?response_type={1}&redirect_uri={2}&app_id={3}&display=popup";
        private static readonly string TokenUrl = "https://graph.facebook.com/oauth/access_token";
        private static readonly string AuthType = "code";
        private static readonly string GrantTypeValue = "authorization_code";

        public FacebookAuthorizer() : base(TokenUrl) 
        {
            _shouldGet = true;
        }

        protected override string GetAuthorizeUrlInternal(string appKey, string redirectUrl, string state)
        {
            return string.Format(AuthorizeUrlTemplate, AuthorizeUrl, AuthType, redirectUrl, appKey);
        }

        protected override NameValueCollection GetTokenRequestParameters(string appKey, string appSecret, string redirectUrl, string code)
        {
            var parameters = new NameValueCollection();
            parameters.Add("grant_type", GrantTypeValue);
            parameters.Add("client_id", appKey);
            parameters.Add("client_secret", appSecret);
            parameters.Add("code", code);
            parameters.Add("redirect_uri", redirectUrl);
            return parameters;
        }

        protected override AccessTokenResponse GetResponseObject(Stream stream, string contentType)
        {
            var serializer = SerializerFactory.CreateSerializer<FacebookAccessToken>(contentType);
            var token = serializer.ReadObject(stream);
            return token.AsAccessTokenResponse();
        }
    }
}
