﻿using OAuth2Authorizer.Framework;
using OAuth2Authorizer.Serialization;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Json;
using System.Text;

namespace OAuth2Authorizer.StackExchange
{
    internal class StackExchangeAuthorizer : AuthorizerBase, IAuthorizer
    {
        private static readonly string AuthorizeUrl = "https://stackexchange.com/oauth";
        private static readonly string AuthorizeUrlTemplate = "{0}?client_id={1}&redirect_uri={2}&state={3}";
        private static readonly string TokenUrl = "https://stackexchange.com/oauth/access_token";
        
        public StackExchangeAuthorizer() : base(TokenUrl) {}

        protected override string GetAuthorizeUrlInternal(string appKey, string redirectUrl, string state)
        {
            return string.Format(AuthorizeUrlTemplate, AuthorizeUrl, appKey, redirectUrl, state);
        }

        protected override NameValueCollection GetTokenRequestParameters(string appKey, string appSecret, string redirectUrl, string code)
        {
            var parameters = new NameValueCollection();
            parameters.Add("code", code);
            parameters.Add("client_id", appKey);
            parameters.Add("client_secret", appSecret);
            parameters.Add("redirect_uri", redirectUrl);
            return parameters;
        }

        protected override AccessTokenResponse GetResponseObject(Stream stream, string contentType)
        {
            var serializer = SerializerFactory.CreateSerializer<StackExchangeAccessToken>(contentType);
            var token = serializer.ReadObject(stream);
            return token.AsAccessTokenResponse();
        }
    }
}
