﻿using OAuth2Authorizer.Framework;
using OAuth2Authorizer.Serialization;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Json;
using System.Text;

namespace OAuth2Authorizer.WordPress
{
    internal class WordPressAuthorizer : AuthorizerBase, IAuthorizer
    {
        private static readonly string AuthorizeUrl = "https://public-api.wordpress.com/oauth2/authorize";
        private static readonly string AuthorizeUrlTemplate = "{0}?response_type={1}&client_id={2}&redirect_uri={3}";
        private static readonly string AuthType = "code";
        private static readonly string TokenUrl = "https://public-api.wordpress.com/oauth2/token";
        private static readonly string GrantTypeValue = "authorization_code";
        
        public WordPressAuthorizer() : base(TokenUrl) {}

        protected override string GetAuthorizeUrlInternal(string appKey, string redirectUrl, string state)
        {
            return string.Format(AuthorizeUrlTemplate, AuthorizeUrl, AuthType, appKey, redirectUrl);
        }

        protected override NameValueCollection GetTokenRequestParameters(string appKey, string appSecret, string redirectUrl, string code)
        {
            var parameters = new NameValueCollection();
            parameters.Add("code", code);
            parameters.Add("grant_type", GrantTypeValue);
            parameters.Add("client_id", appKey);
            parameters.Add("client_secret", appSecret);
            parameters.Add("redirect_uri", redirectUrl);
            return parameters;
        }

        protected override AccessTokenResponse GetResponseObject(Stream stream, string contentType)
        {
            var serializer = SerializerFactory.CreateSerializer<WordPressAccessToken>(contentType);
            var token = serializer.ReadObject(stream);
            return token.AsAccessTokenResponse();
        }
    }
}
